@@ -9,6 +9,7 @@ from skimage.measure import perimeter
 from matplotlib import cm
 from scipy import stats
 import seaborn as sns
+from torch import clamp_min
 from NaroNet.utils.parallel_process import parallel_process
 from tifffile.tifffile import imwrite
 import pandas as pd
@@ -16,6 +17,98 @@ import gc
 import itertools
 from skimage import filters
 import time 
+from openTSNE import TSNE
+from sklearn.preprocessing import StandardScaler
+
+
+def visualize_tsne_phenotypes(dataset, IndexAndClass,clusters):
+
+    def load_embeddings(patch_dir):
+        
+        # Initialize features    
+        features = []
+        features_idx = []
+
+        # Extract patches and save it into a graph
+        for subject in IndexAndClass:                                    
+            
+            # Load image 
+            subject_feats = np.load(patch_dir + subject[0] + '.npy')
+
+            # Store feats
+            features.append(subject_feats)
+            
+            # Store subject idx
+            features_idx.append([subject[1]]*subject_feats.shape[0])
+
+        # Store in one file all features
+        features = np.concatenate(features,axis=0)
+        features_idx = np.concatenate(features_idx,axis=0)    
+        return features, features_idx
+    
+    def tsne_with_pheno_thrshold(dataset,features_idx,features,pheno_assignmnents):
+        
+        # Select number of patches
+        num_val =min(10000000,features_idx.shape[0])     
+        features_idx = features_idx[::int(features_idx.shape[0]/num_val)]
+        features = features[::int(features.shape[0]/num_val),:]
+        pheno_assignmnents = pheno_assignmnents[::int(features.shape[0]/num_val),:]
+        
+        # Reorder samples
+        sel_vals = np.random.randint(0,high=features.shape[0],size=features.shape[0])
+        features = features[sel_vals,:]
+        features_idx = features_idx[sel_vals]
+        pheno_assignmnents = pheno_assignmnents[sel_vals,:]
+        
+        # Normalize features
+        scaled_features = StandardScaler().fit_transform(features[:,2:])
+
+        # Create UMAP
+        embedding = TSNE(n_jobs=8,random_state=3, verbose=True).fit(scaled_features)
+
+        # Show each phenotype
+        for ph in range(pheno_assignmnents.shape[1]):
+
+            # Create and save scatter plot figure
+            plt.close()
+            fig, ax = plt.subplots()   
+            scatter = ax.scatter(embedding[:, 0],embedding[:, 1],s=0.5,c=pheno_assignmnents[:,ph],vmin=0,vmax=1,cmap='jet',edgecolors='none')        
+            plt.title('TSNE of the dataset Phenoptype - '+str(ph+1), fontsize=24)    
+            plt.colorbar(scatter)
+            plt.savefig(dataset.bioInsights_dir_cell_types_Pheno+'TSNE_embeddings_Ph_'+str(ph+1)+'.png',dpi=600)
+
+        pheno_assignmnents_75 = np.zeros(pheno_assignmnents.shape[0])
+        for ph in range(pheno_assignmnents.shape[1]):
+            thr = np.percentile(pheno_assignmnents[pheno_assignmnents.argmax(1)==ph,ph],75)
+            pheno_assignmnents_75[np.logical_and(pheno_assignmnents.argmax(1)==ph,pheno_assignmnents[:,ph]>=thr)] = ph+1
+        plt.close()
+        fig, ax = plt.subplots()   
+        scatter = ax.scatter(embedding[:, 0],embedding[:, 1],s=0.5,c=pheno_assignmnents_75,cmap='jet',edgecolors='none')        
+        plt.title('TSNE of the dataset ALL_Phenotypes Q4', fontsize=24)    
+        plt.colorbar(scatter)
+        plt.savefig(dataset.bioInsights_dir_cell_types_Pheno+'TSNE_embeddings_ALL_Phenotypes_Q4.png',dpi=600)
+
+
+    pat_num = int(len(IndexAndClass)*dataset.args['TSNE_Perc_Pat'])
+    IndexAndClass = IndexAndClass[:pat_num]
+
+    # Directory to load
+    patch_dir = dataset.root+'Patch_Contrastive_Learning/Image_Patch_Representation/'       
+
+    # Load data
+    features, features_idx = load_embeddings(patch_dir)
+
+    # Load pheno assignments
+    pheno_assignmnents = []
+    for subject_idx, subject_info in enumerate(IndexAndClass):              
+        _, cell_type_assignment = load_cell_types_assignments(dataset, 0, subject_info, 0, clusters[0], [])
+        pheno_assignmnents.append(cell_type_assignment[:(features_idx==subject_info[1]).sum(),:])
+    pheno_assignmnents = np.concatenate(pheno_assignmnents,axis=0)   
+        
+    # Create Tsne 
+    tsne_with_pheno_thrshold(dataset,features_idx,features,pheno_assignmnents)
+
+    return
 
 def load_cell_types_assignments(dataset, cell_type_idx, subject_info,subgraph_idx,n_cell_types, prev_cell_type_assignment):
     """
