@@ -6,7 +6,8 @@ import torch.nn.functional as F
 def calc_contrast_loss(features,assignment_orig,device):
 
     # Use a batch size of 1000
-    nbatch = max(min(10,assignment_orig.shape[0]),assignment_orig.shape[1])
+    nbatch = max(min(50,int(assignment_orig.shape[0]/assignment_orig.shape[1]*0.01),assignment_orig.shape[0]),assignment_orig.shape[1])
+    n_neighb = 100
 
     # Calculate feature similarities
     features = F.normalize(features.squeeze(), dim=1)    
@@ -16,10 +17,10 @@ def calc_contrast_loss(features,assignment_orig,device):
     for ph in range(assignment_orig.shape[1]):
         sel_vals = assignment_orig[:,ph].topk(nbatch,0)[1]       
         feats = torch.matmul(features[sel_vals], features.T)        
-        top_elem = (feats.topk(100,1)[1]).reshape(100*nbatch)
-        assignment = assignment_orig[sel_vals][top_elem]
-        positives = torch.exp((torch.matmul(assignment,assignment.T)/temp))
+        top_elem = (feats.topk(n_neighb,1)[1]).reshape(100*nbatch)        
+        positives = torch.exp((torch.matmul(assignment_orig[top_elem],assignment_orig[top_elem].T)/temp))
         loss += -torch.log(positives).mean()*temp
+        print((assignment_orig[top_elem].argmax(1)==ph).sum()/top_elem.shape[0])
 
     return loss/assignment_orig.shape[1]
 
