@@ -178,12 +178,23 @@ class ContrastiveLearningDataset_PCL(Dataset):
         # Flip lr
         if random.random()<0.5:
             for c in range(aug.shape[2]):
-                aug[:,:,c] = np.flip(aug[:,:,c],0)
+                aug[:,:,c] = np.fliplr(aug[:,:,c])
     
         # Flip ud
         if random.random()<0.5:
             for c in range(aug.shape[2]):
-                aug[:,:,c] = np.flip(aug[:,:,c],1)
+                aug[:,:,c] = np.flipud(aug[:,:,c])
+
+        # Flip ud
+        if random.random()<0.5:
+            for c in range(aug.shape[2]):
+                aug[:,:,c] = np.rot90(aug[:,:,c],-1)
+
+        # Flip ud
+        if random.random()<0.5:
+            for c in range(aug.shape[2]):
+                aug[:,:,c] = np.rot90(aug[:,:,c],1)
+
 
         # RGB
         if aug.shape[2]==3:
